/***
   NAME
     chinook2.c
***/

/*==========================================================================
 *			INCLUDING THE HEADER FILE
 *==========================================================================
 */
#include  "escbox.h"

/*
 *==========================================================================
 *		LABELLING ENVIRONMENT AND I-STATE VARIABLES
 *==========================================================================
 */

#define time 		env[0]
#define food 		env[1]

#define	age		i_state(0)
#define	size		i_state(1)

#define	deathrate	i_const(0)
#define	fecundity	i_const(1)
#define	funresp	        i_const(2)
#define	intakerate      i_const(3)
#define	p               i_const(4)


/*
 *==========================================================================
 *		DEFINING AND LABELLING CONSTANTS AND PARAMETERS
 *==========================================================================
 */

#define S0		parameter[0]
#define SM		parameter[1]
#define GAMMA           parameter[2]
#define BETA		parameter[3]
#define MU0		parameter[4]
#define MUINF		parameter[5]
#define SMU		parameter[6]
#define ALPHA		parameter[7]
#define FH              parameter[8]
#define RHO             parameter[9]
#define K               parameter[10]
#define P0              parameter[11]
#define P1              parameter[12]
#define P2              parameter[13]
#define P3              parameter[14]

#define YEAR            100.0
#define SMALL           0.01


/*
 *==========================================================================
 * USER INITIALIZATION ROUTINE ALLOWS OPERATIONS ON INITIAL POPULATIONS
 *==========================================================================
 */

void	UserInit( int argc, char **argv, double *env,  population *pop)
{

  return;
}

/*
 *==========================================================================
 *	SPECIFICATION OF THE NUMBER AND VALUES OF BOUNDARY POINTS
 *==========================================================================
 */

void	SetBpointNo(double *env, population *pop, int *bpoint_no)
{
  bpoint_no[0]=1;
  
  return;
}

/*==========================================================================*/

void	SetBpoints(double *env, population *pop, population *bpoints)
{
  bpoints[0][0][age]  = 0.0; 
  bpoints[0][0][size] = S0; 
  
  return;
}


/*
 *==========================================================================
 *			SPECIFICATION OF DERIVATIVES
 *==========================================================================
 */

void	Gradient(double *env,     population *pop,     population *ofs,
		 double *envgrad, population *popgrad, population *ofsgrad,
		 population *bpoints)
{
  double		SINF, totcons;
  register int		i;
  int                   a;

  for(i=0, totcons=0.0; i<cohort_no[0]; i++)	 
    {
      popIDcard[0][i][deathrate]  = MUINF + (MUINF-MU0) *
	exp(-pop[0][i][size]/SMU);
      popIDcard[0][i][funresp]    = food/(food+FH);
      popIDcard[0][i][intakerate] = ALPHA*pop[0][i][size]*pop[0][i][size]*
	popIDcard[0][i][funresp];
      popIDcard[0][i][fecundity]  = BETA*pop[0][i][size];
      
      a = (int)floor((pop[0][i][age]+SMALL)/YEAR);
      switch(a)
	{
	case 0:
	  popIDcard[0][i][p]          = P0;
	  break;
	case 1:
	  popIDcard[0][i][p]          = P1;
	  break;
	case 2:
	  popIDcard[0][i][p]          = P2;
	  break;
	case 3:
	  popIDcard[0][i][p]          = P3;
	  break;
	}
      
      totcons += popIDcard[0][i][intakerate]*pop[0][i][number];
      
      SINF = SM * popIDcard[0][i][funresp];
      
      popgrad[0][i][size] = GAMMA*(SINF - pop[0][i][size]);
      
    }
  
  for(i=0; i<cohort_no[0]; i++)			
    {						
      popgrad[0][i][number] = -popIDcard[0][i][deathrate]*pop[0][i][number];
      popgrad[0][i][age]    = 1.0;
    }

  envgrad[0] = 1.0;
  envgrad[1] = RHO*(K - food) - totcons;

  return;
}



/*
 *==========================================================================
 *		SPECIFICATION OF BETWEEN COHORT CYCLE DYNAMICS
 *==========================================================================
 */

void	InstantDynamics(double *env, population *pop, population *ofs)
{
  double                date, juveniles;
  register int		i;
  int                   a;

  /* What is the date? */
  date = fmod(time, YEAR);
  if (date != 0.0)
    return;                
  /* Return if within season, otherwise opportunity to reproduce */
 
  for (i=0, juveniles=0.0; i<cohort_no[0]; i++)
    {
      a = (int)floor((pop[0][i][age]+SMALL)/YEAR);
      fprintf(stderr, "t=%f, age of cohort %d is %d\n", time,i,a);

      fprintf(stderr, "reproduction at time %f\n",time);
      juveniles += popIDcard[0][i][p]*pop[0][i][number]*pop[0][i][fecundity];
      pop[0][i][number] = (1-popIDcard[0][i][p])*pop[0][i][number];;
    }  
  
  ofs[0][0][number]             = juveniles;
  ofs[0][0][age]                = 0.0;
  ofs[0][0][size]               = S0;
  ofsIDcard[0][0][deathrate]    =  MUINF + (MUINF-MU0) *
    exp(-pop[0][i][size]/SMU);
  
  return;
}



/*
 *==========================================================================
 *			SPECIFICATION OF OUTPUT VARIABLES
 *==========================================================================
 */

void	DefineOutput(double *env, population *pop, double *output)
{

  double                totpop, age0, age1, age2, age3, age4;
  register int		i;	
  int                   a;
  
  for(i=0, totpop=0.0; i<cohort_no[0]; i++)
    {
      totpop += pop[0][i][number];
    }	

  /* count individuals in three age classes */
  age0=0.0;
  age1=0.0;
  age2=0.0;
  age3=0.0;
  age4=0.0;
  for(i=0; i<cohort_no[0]; i++)		      
    {
      a = (int)floor((pop[0][i][age]+SMALL)/YEAR);
      switch(a)
	{
	case 0:
	  age0 = pop[0][i][number];
	  break;
	case 1:
	  age1 = pop[0][i][number];
	  break;
	case 2:
	  age2 = pop[0][i][number];
	  break;
	case 3:
	  age3 = pop[0][i][number];
	  break;
	case 4:
	  age4 = pop[0][i][number];
	  break;
	}
    }

  output[0] = food;
  output[1] = totpop;
  output[2] = age0;
  output[3] = age1;
  output[4] = age2;
  output[5] = age3;
  output[6] = cohort_no[0];
  
  return;
}
/*==========================================================================*/
